@@ -4,18 +4,20 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
+from abc import abstractmethod
 import binascii
 import sys
 
-from bitstring import BitArray, BitStream, CreationError
+from bitstring import BitArray, BitStream, Bits, CreationError
 
+from espefuse.efuse.mem_definition_base import EfuseBlocksBase, EfuseRegistersBase
 import esptool
+from esptool.logger import log
 
 from . import util
-from typing import List
 
 
-class CheckArgValue(object):
+class CheckArgValue:
     def __init__(self, efuses, name):
         self.efuses = efuses
         self.name = name
@@ -26,7 +28,7 @@ class CheckArgValue(object):
                 new_value = 1 if new_value is None else int(new_value, 0)
                 if new_value != 1:
                     raise esptool.FatalError(
-                        "New value is not accepted for efuse '{}' "
+                        "New value is not accepted for eFuse '{}' "
                         "(will always burn 0->1), given value={}".format(
                             efuse.name, new_value
                         )
@@ -46,7 +48,7 @@ class CheckArgValue(object):
                 else:
                     if new_value is None:
                         raise esptool.FatalError(
-                            "New value required for efuse '{}' (given None)".format(
+                            "New value required for eFuse '{}' (given None)".format(
                                 efuse.name
                             )
                         )
@@ -59,20 +61,20 @@ class CheckArgValue(object):
             elif efuse.efuse_type.startswith("bytes"):
                 if new_value is None:
                     raise esptool.FatalError(
-                        "New value required for efuse '{}' (given None)".format(
+                        "New value required for eFuse '{}' (given None)".format(
                             efuse.name
                         )
                     )
                 if len(new_value) * 8 != efuse.bitarray.len:
                     raise esptool.FatalError(
-                        "The length of efuse '{}' ({} bits) "
+                        "The length of eFuse '{}' ({} bits) "
                         "(given len of the new value= {} bits)".format(
                             efuse.name, efuse.bitarray.len, len(new_value) * 8
                         )
                     )
             else:
                 raise esptool.FatalError(
-                    "The '{}' type for the '{}' efuse is not supported yet.".format(
+                    "The '{}' type for the '{}' eFuse is not supported yet.".format(
                         efuse.efuse_type, efuse.name
                     )
                 )
@@ -83,8 +85,12 @@ class CheckArgValue(object):
         return check_arg_value(efuse, new_value)
 
 
-class EfuseProtectBase(object):
+class EfuseProtectBase:
     # This class is used by EfuseBlockBase and EfuseFieldBase
+    read_disable_bit: int | list[int] | None
+    write_disable_bit: int | None
+    parent: "EspEfusesBase"
+    name: str
 
     def get_read_disable_mask(self, blk_part=None):
         """Returns mask of read protection bits
@@ -108,35 +114,48 @@ class EfuseProtectBase(object):
         # On the C2 chip, BLOCK_KEY0 has two read protection bits [0, 1].
         return bin(self.get_read_disable_mask()).count("1")
 
-    def is_readable(self, blk_part=None):
-        """Return true if the efuse is readable by software"""
+    def is_readable(self, blk_part: int | None = None) -> bool:
+        """Check if the eFuse is readable by software
+
+        Args:
+            blk_part: The part of the block to check.
+                If None, check all parts.
+
+        Returns:
+            bool: True if the eFuse is readable by software
+        """
         num_bit = self.read_disable_bit
         if num_bit is None:
             return True  # read cannot be disabled
-        return (self.parent["RD_DIS"].get() & self.get_read_disable_mask(blk_part)) == 0
+        return (self.parent["RD_DIS"].get() & self.get_read_disable_mask(blk_part)) == 0  # type: ignore
 
     def disable_read(self):
         num_bit = self.read_disable_bit
         if num_bit is None:
-            raise esptool.FatalError("This efuse cannot be read-disabled")
+            raise esptool.FatalError("This eFuse cannot be read-disabled")
         if not self.parent["RD_DIS"].is_writeable():
             raise esptool.FatalError(
-                "This efuse cannot be read-disabled due the to RD_DIS field is "
+                "This eFuse cannot be read-disabled due to the RD_DIS field being "
                 "already write-disabled"
             )
         self.parent["RD_DIS"].save(self.get_read_disable_mask())
 
-    def is_writeable(self):
+    def is_writeable(self) -> bool:
+        """Check if the eFuse is writeable by software
+
+        Returns:
+            bool: True if the eFuse is writeable by software
+        """
         num_bit = self.write_disable_bit
         if num_bit is None:
             return True  # write cannot be disabled
-        return (self.parent["WR_DIS"].get() & (1 << num_bit)) == 0
+        return (self.parent["WR_DIS"].get() & (1 << num_bit)) == 0  # type: ignore
 
     def disable_write(self):
         num_bit = self.write_disable_bit
         if not self.parent["WR_DIS"].is_writeable():
             raise esptool.FatalError(
-                "This efuse cannot be write-disabled due to the WR_DIS field is "
+                "This eFuse cannot be write-disabled due to the WR_DIS field being "
                 "already write-disabled"
             )
         self.parent["WR_DIS"].save(1 << num_bit)
@@ -144,7 +163,7 @@ class EfuseProtectBase(object):
     def check_wr_rd_protect(self):
         if not self.is_readable():
             error_msg = "\t{} is read-protected.".format(self.name)
-            "The written value can not be read, the efuse/block looks as all 0.\n"
+            "The written value can not be read, the eFuse/block looks as all 0.\n"
             error_msg += "\tBurn in this case may damage an already written value."
             self.parent.print_error_msg(error_msg)
         if not self.is_writeable():
@@ -155,26 +174,26 @@ class EfuseProtectBase(object):
 
 
 class EfuseBlockBase(EfuseProtectBase):
-    def __init__(self, parent, param, skip_read=False):
-        self.parent = parent
-        self.name = param.name
-        self.alias = param.alias
-        self.id = param.id
-        self.rd_addr = param.rd_addr
-        self.wr_addr = param.wr_addr
-        self.write_disable_bit = param.write_disable_bit
-        self.read_disable_bit = param.read_disable_bit
-        self.len = param.len
-        self.key_purpose_name = param.key_purpose
-        bit_block_len = self.get_block_len() * 8
-        self.bitarray = BitStream(bit_block_len)
+    def __init__(self, parent: "EspEfusesBase", param, skip_read: bool = False) -> None:
+        self.parent: EspEfusesBase = parent
+        self.name: str = param.name
+        self.alias: list[str] = param.alias
+        self.id: int = param.id
+        self.rd_addr: int = param.rd_addr
+        self.wr_addr: int = param.wr_addr
+        self.write_disable_bit: int | None = param.write_disable_bit
+        self.read_disable_bit: int | list[int] | None = param.read_disable_bit
+        self.len: int = param.len
+        self.key_purpose_name: str | None = param.key_purpose
+        bit_block_len: int = self.get_block_len() * 8
+        self.bitarray: BitStream = BitStream(bit_block_len)
         self.bitarray.set(0)
-        self.wr_bitarray = BitStream(bit_block_len)
+        self.wr_bitarray: BitStream = BitStream(bit_block_len)
         self.wr_bitarray.set(0)
-        self.fail = False
-        self.num_errors = 0
+        self.fail: bool = False
+        self.num_errors: int = 0
         if self.id == 0:
-            self.err_bitarray = BitStream(bit_block_len)
+            self.err_bitarray: BitStream | None = BitStream(bit_block_len)
             self.err_bitarray.set(0)
         else:
             self.err_bitarray = None
@@ -182,6 +201,10 @@ class EfuseBlockBase(EfuseProtectBase):
         if not skip_read:
             self.read()
 
+    @abstractmethod
+    def apply_coding_scheme(self):
+        pass
+
     def get_block_len(self):
         coding_scheme = self.get_coding_scheme()
         if coding_scheme == self.parent.REGS.CODING_SCHEME_NONE:
@@ -191,9 +214,7 @@ class EfuseBlockBase(EfuseProtectBase):
         elif coding_scheme == self.parent.REGS.CODING_SCHEME_RS:
             return self.len * 4
         else:
-            raise esptool.FatalError(
-                "Coding scheme (%d) not supported" % (coding_scheme)
-            )
+            raise esptool.FatalError(f"Coding scheme ({coding_scheme}) not supported")
 
     def get_coding_scheme(self):
         if self.id == 0:
@@ -207,10 +228,10 @@ class EfuseBlockBase(EfuseProtectBase):
         else:
             return self.wr_bitarray.bytes
 
-    def get(self, from_read=True):
-        self.get_bitstring(from_read=from_read)
+    def get(self, from_read: bool = True) -> BitStream:
+        return self.get_bitstring(from_read=from_read)
 
-    def get_bitstring(self, from_read=True):
+    def get_bitstring(self, from_read: bool = True) -> BitStream:
         if from_read:
             return self.bitarray
         else:
@@ -232,7 +253,7 @@ class EfuseBlockBase(EfuseProtectBase):
         words = self.get_words()
         data = BitArray()
         for word in reversed(words):
-            data.append("uint:32=%d" % word)
+            data.append(f"uint:32={word}")
         self.bitarray.overwrite(data, pos=0)
         if print_info:
             self.print_block(self.bitarray, "read_regs")
@@ -240,14 +261,14 @@ class EfuseBlockBase(EfuseProtectBase):
     def print_block(self, bit_string, comment, debug=False):
         if self.parent.debug or debug:
             bit_string.pos = 0
-            print(
-                "%-15s (%-16s) [%-2d] %s:"
-                % (self.name, " ".join(self.alias)[:16], self.id, comment),
+            log.print(
+                f"{self.name:<15s} ({' '.join(self.alias)[:16]:<16s}) "
+                f"[{self.id:<2d}] {comment}:",
                 " ".join(
                     [
-                        "%08x" % word
+                        f"{word:08x}"
                         for word in bit_string.readlist(
-                            "%d*uint:32" % (bit_string.len / 32)
+                            f"{int(bit_string.len / 32)}*uint:32"
                         )[::-1]
                     ]
                 ),
@@ -258,17 +279,17 @@ class EfuseBlockBase(EfuseProtectBase):
         if wr_data.all(False):
             # nothing to burn
             if self.parent.debug:
-                print("[{:02}] {:20} nothing to burn".format(self.id, self.name))
+                log.print("[{:02}] {:20} nothing to burn".format(self.id, self.name))
             return False
         if len(wr_data.bytes) != len(self.bitarray.bytes):
             raise esptool.FatalError(
-                "Data does not fit: the block%d size is %d bytes, data is %d bytes"
-                % (self.id, len(self.bitarray.bytes), len(wr_data.bytes))
+                f"Data does not fit: block{self.id} size "
+                f"{len(self.bitarray.bytes)} bytes, data {len(wr_data.bytes)} bytes"
             )
         self.check_wr_rd_protect()
 
         if self.get_bitstring().all(False):
-            print(
+            log.print(
                 "[{:02}] {:20} is empty, will burn the new value".format(
                     self.id, self.name
                 )
@@ -276,18 +297,18 @@ class EfuseBlockBase(EfuseProtectBase):
         else:
             # the written block in chip is not empty
             if self.get_bitstring() == wr_data:
-                print(
+                log.print(
                     "[{:02}] {:20} is already written the same value, "
                     "continue with EMPTY_BLOCK".format(self.id, self.name)
                 )
                 wr_data.set(0)
             else:
-                print("[{:02}] {:20} is not empty".format(self.id, self.name))
-                print("\t(written ):", self.get_bitstring())
-                print("\t(to write):", wr_data)
+                log.print("[{:02}] {:20} is not empty".format(self.id, self.name))
+                log.print("\t(written ):", self.get_bitstring())
+                log.print("\t(to write):", wr_data)
                 mask = self.get_bitstring() & wr_data
                 if mask == wr_data:
-                    print(
+                    log.print(
                         "\tAll wr_data bits are set in the written block, "
                         "continue with EMPTY_BLOCK."
                     )
@@ -295,49 +316,47 @@ class EfuseBlockBase(EfuseProtectBase):
                 else:
                     coding_scheme = self.get_coding_scheme()
                     if coding_scheme == self.parent.REGS.CODING_SCHEME_NONE:
-                        print("\t(coding scheme = NONE)")
+                        log.print("\t(coding scheme = NONE)")
                     elif coding_scheme == self.parent.REGS.CODING_SCHEME_RS:
-                        print("\t(coding scheme = RS)")
+                        log.print("\t(coding scheme = RS)")
                         error_msg = (
-                            "\tBurn into %s is forbidden "
-                            "(RS coding scheme does not allow this)." % (self.name)
+                            f"\tBurn into {self.name} is forbidden "
+                            "(RS coding scheme does not allow this)."
                         )
                         self.parent.print_error_msg(error_msg)
                     elif coding_scheme == self.parent.REGS.CODING_SCHEME_34:
-                        print("\t(coding scheme = 3/4)")
+                        log.print("\t(coding scheme = 3/4)")
                         data_can_not_be_burn = False
                         for i in range(0, self.get_bitstring().len, 6 * 8):
                             rd_chunk = self.get_bitstring()[i : i + 6 * 8 :]
                             wr_chunk = wr_data[i : i + 6 * 8 :]
                             if rd_chunk.any(True):
                                 if wr_chunk.any(True):
-                                    print(
-                                        "\twritten chunk [%d] and wr_chunk "
-                                        "are not empty. " % (i // (6 * 8)),
+                                    log.print(
+                                        f"\twritten chunk [{i // (6 * 8)}] and wr_chunk"
+                                        " are not empty. ",
                                         end="",
                                     )
                                     if rd_chunk == wr_chunk:
-                                        print(
+                                        log.print(
                                             "wr_chunk == rd_chunk. "
                                             "Continue with empty chunk."
                                         )
                                         wr_data[i : i + 6 * 8 :].set(0)
                                     else:
-                                        print("wr_chunk != rd_chunk. Can not burn.")
-                                        print("\twritten ", rd_chunk)
-                                        print("\tto write", wr_chunk)
+                                        log.print("wr_chunk != rd_chunk. Can not burn.")
+                                        log.print("\twritten ", rd_chunk)
+                                        log.print("\tto write", wr_chunk)
                                         data_can_not_be_burn = True
                         if data_can_not_be_burn:
                             error_msg = (
-                                "\tBurn into %s is forbidden "
-                                "(3/4 coding scheme does not allow this)." % (self.name)
+                                f"\tBurn into {self.name} is forbidden "
+                                "(3/4 coding scheme does not allow this)."
                             )
                             self.parent.print_error_msg(error_msg)
                     else:
                         raise esptool.FatalError(
-                            "The coding scheme ({}) is not supported".format(
-                                coding_scheme
-                            )
+                            f"The coding scheme ({coding_scheme}) is not supported."
                         )
 
     def save(self, new_data):
@@ -351,16 +370,14 @@ class EfuseBlockBase(EfuseProtectBase):
         # *[x] - means a byte.
         data = BitStream(bytes=new_data[::-1], length=len(new_data) * 8)
         if self.parent.debug:
-            print(
-                "\twritten : {} ->\n\tto write: {}".format(self.get_bitstring(), data)
-            )
+            log.print(f"\twritten : {self.get_bitstring()} ->\n\tto write: {data}")
         self.wr_bitarray.overwrite(self.wr_bitarray | data, pos=0)
 
     def burn_words(self, words):
         for burns in range(3):
             self.parent.efuse_controller_setup()
             if self.parent.debug:
-                print("Write data to BLOCK%d" % (self.id))
+                log.print(f"Write data to BLOCK{self.id}")
             write_reg_addr = self.wr_addr
             for word in words:
                 # for ep32s2: using EFUSE_PGM_DATA[0..7]_REG for writing data
@@ -371,7 +388,7 @@ class EfuseBlockBase(EfuseProtectBase):
                 #   each block has the special regs EFUSE_BLK[0..3]_WDATA[0..7]_REG
                 #   for writing data
                 if self.parent.debug:
-                    print("Addr 0x%08x, data=0x%08x" % (write_reg_addr, word))
+                    log.print(f"Addr {write_reg_addr:10x}, data={word:10x}")
                 self.parent.write_reg(write_reg_addr, word)
                 write_reg_addr += 4
 
@@ -380,10 +397,9 @@ class EfuseBlockBase(EfuseProtectBase):
                 self.parent.efuse_read()
                 self.parent.get_coding_scheme_warnings(silent=True)
                 if self.fail or self.num_errors:
-                    print(
-                        "Error in BLOCK%d, re-burn it again (#%d), to fix it. "
-                        "fail_bit=%d, num_errors=%d"
-                        % (self.id, burns, self.fail, self.num_errors)
+                    log.print(
+                        f"Error in BLOCK{self.id}, re-burn it again (#{burns}) to fix."
+                        f" fail_bit={self.fail}, num_errors={self.num_errors}"
                     )
                     break
             if not self.fail and self.num_errors == 0:
@@ -391,13 +407,15 @@ class EfuseBlockBase(EfuseProtectBase):
                 if self.wr_bitarray & self.bitarray != self.wr_bitarray:
                     # if the required bits are not set then we need to re-burn it again.
                     if burns < 2:
-                        print(
-                            f"\nRepeat burning BLOCK{self.id} (#{burns + 2}) because not all bits were set"
+                        log.print(
+                            f"\nRepeat burning BLOCK{self.id} (#{burns + 2}) "
+                            "because not all bits were set"
                         )
                         continue
                     else:
-                        print(
-                            f"\nAfter {burns + 1} attempts, the required data was not set to BLOCK{self.id}"
+                        log.print(
+                            f"\nAfter {burns + 1} attempts, the required data was not "
+                            f"set to BLOCK{self.id}"
                         )
                 break
 
@@ -412,29 +430,28 @@ class EfuseBlockBase(EfuseProtectBase):
         self.burn_words(words)
         self.read()
         if not self.is_readable():
-            print(
-                "{} ({}) is read-protected. "
-                "Read back the burn value is not possible.".format(
-                    self.name, self.alias
-                )
+            log.print(
+                f"{self.name} ({self.alias}) is read-protected. "
+                "Read back the burn value is not possible."
             )
             if self.bitarray.all(False):
-                print("Read all '0'")
+                log.print("Read all '0'")
             else:
                 # Should never happen
                 raise esptool.FatalError(
-                    "The {} is read-protected but not all '0' ({})".format(
-                        self.name, self.bitarray.hex
-                    )
+                    f"The {self.name} is read-protected but not all '0' "
+                    f"({self.bitarray.hex})"
                 )
         else:
             if self.wr_bitarray == self.bitarray:
-                print("BURN BLOCK%-2d - OK (write block == read block)" % self.id)
+                log.print(f"BURN BLOCK{self.id:<2d} - OK (write block == read block)")
             elif (
                 self.wr_bitarray & self.bitarray == self.wr_bitarray
                 and self.bitarray & before_burn_bitarray == before_burn_bitarray
             ):
-                print("BURN BLOCK%-2d - OK (all write block bits are set)" % self.id)
+                log.print(
+                    f"BURN BLOCK{self.id:<2d} - OK (all write block bits are set)"
+                )
             else:
                 # Happens only when an efuse is written and read-protected
                 # in one command
@@ -444,27 +461,46 @@ class EfuseBlockBase(EfuseProtectBase):
                 # raise error only for other blocks
                 if self.id != 0:
                     raise esptool.FatalError(
-                        "Burn {} ({}) was not successful".format(self.name, self.alias)
+                        f"Burn {self.name} ({self.alias}) was not successful."
                     )
         self.wr_bitarray.set(0)
 
 
-class EspEfusesBase(object):
+class EspEfusesBase:
     """
     Wrapper object to manage the efuse fields in a connected ESP bootloader
     """
 
-    _esp = None
-    blocks: List[EfuseBlockBase] = []
-    efuses: List = []
+    _esp: esptool.ESPLoader
+    blocks: list[EfuseBlockBase] = []
+    efuses: list = []
     coding_scheme = None
     force_write_always = None
-    batch_mode_cnt = 0
-    postpone = False
+    batch_mode_cnt: int = 0
+    postpone: bool = False
+    BURN_BLOCK_DATA_NAMES: list[str] = []
+    REGS: type[EfuseRegistersBase]
+    Blocks: EfuseBlocksBase
+
+    def __init__(
+        self,
+        esp: esptool.ESPLoader,
+        skip_connect: bool = False,
+        debug: bool = False,
+        do_not_confirm: bool = False,
+        extend_efuse_table: None = None,
+    ) -> None:
+        self._esp = esp
+        self.debug = debug
+        self.do_not_confirm = do_not_confirm
 
     def __iter__(self):
         return self.efuses.__iter__()
 
+    @abstractmethod
+    def __getitem__(self, efuse_name):
+        pass
+
     def get_crystal_freq(self):
         return self._esp.get_crystal_freq()
 
@@ -484,12 +520,30 @@ class EspEfusesBase(object):
     def efuse_controller_setup(self):
         pass
 
+    @abstractmethod
+    def write_efuses(self, block):
+        pass
+
+    @abstractmethod
+    def efuse_read(self):
+        pass
+
+    @abstractmethod
+    def read_coding_scheme(self):
+        pass
+
     def reconnect_chip(self, esp):
-        print("Re-connecting...")
+        log.print("Re-connecting...")
         baudrate = esp._port.baudrate
         port = esp._port.port
+        connect_mode = (
+            "usb-reset"
+            if esp._get_pid() == esp.USB_JTAG_SERIAL_PID
+            else "default-reset"
+        )
+        log.print(f"Port: {port}, Baudrate: {baudrate}, Connect mode: {connect_mode}")
         esp._port.close()
-        return esptool.cmds.detect_chip(port, baudrate)
+        return esptool.detect_chip(port, baudrate, connect_mode)
 
     def get_index_block_by_name(self, name):
         for block in self.blocks:
@@ -545,8 +599,8 @@ class EspEfusesBase(object):
 
         if any(value != 0 for value in postpone_efuses.values()):
             if self.debug:
-                print("These BLOCK0 efuses will be burned later at the very end:")
-                print(postpone_efuses)
+                log.print("These BLOCK0 eFuses will be burned later at the very end:")
+                log.print(postpone_efuses)
             # exclude these efuses from the first burn (postpone them till the end).
             for key_name in postpone_efuses.keys():
                 self[key_name].reset()
@@ -554,20 +608,20 @@ class EspEfusesBase(object):
 
     def recover_postponed_efuses_from_block0_to_burn(self, postpone_efuses):
         if any(value != 0 for value in postpone_efuses.values()):
-            print("Burn postponed efuses from BLOCK0.")
+            log.print("Burn postponed eFuses from BLOCK0.")
             for key_name in postpone_efuses.keys():
                 self[key_name].save(postpone_efuses[key_name])
 
-    def burn_all(self, check_batch_mode=False):
+    def burn_all(self, check_batch_mode: bool = False) -> bool:
         if check_batch_mode:
             if self.batch_mode_cnt != 0:
-                print(
+                log.print(
                     "\nBatch mode is enabled, "
                     "the burn will be done at the end of the command."
                 )
                 return False
-        print("\nCheck all blocks for burn...")
-        print("idx, BLOCK_NAME,          Conclusion")
+        log.print("\nCheck all blocks for burn...")
+        log.print("idx, BLOCK_NAME,          Conclusion")
         have_wr_data_for_burn = False
         for block in self.blocks:
             block.check_wr_data()
@@ -576,7 +630,7 @@ class EspEfusesBase(object):
             ):
                 have_wr_data_for_burn = True
         if not have_wr_data_for_burn:
-            print("Nothing to burn, see messages above.")
+            log.print("Nothing to burn, see messages above.")
             return True
         EspEfusesBase.confirm("", self.do_not_confirm)
 
@@ -588,16 +642,16 @@ class EspEfusesBase(object):
                 block.num_errors and block.num_errors > old_num_errors
             ):
                 if postponed_efuses:
-                    print("The postponed efuses were not burned due to an error.")
-                    print("\t1. Try to fix a coding error by this cmd:")
-                    print("\t   'espefuse.py check_error --recovery'")
+                    log.print("The postponed eFuses were not burned due to an error.")
+                    log.print("\t1. Try to fix a coding error by this cmd:")
+                    log.print("\t   'espefuse check-error --recovery'")
                     command_string = " ".join(
                         f"{key} {value}"
                         for key, value in postponed_efuses.items()
                         if value.any(True)
                     )
-                    print("\t2. Then run the cmd to burn all postponed efuses:")
-                    print(f"\t   'espefuse.py burn_efuse {command_string}'")
+                    log.print("\t2. Then run the cmd to burn all postponed eFuses:")
+                    log.print(f"\t   'espefuse burn-efuse {command_string}'")
 
                 raise esptool.FatalError("Error(s) were detected in eFuses")
 
@@ -615,7 +669,7 @@ class EspEfusesBase(object):
                 self.recover_postponed_efuses_from_block0_to_burn(postponed_efuses)
                 burn_block(block, postponed_efuses)
 
-        print("Reading updated efuses...")
+        log.print("Reading updated eFuses...")
         self.read_coding_scheme()
         self.read_blocks()
         self.update_efuses()
@@ -623,17 +677,18 @@ class EspEfusesBase(object):
 
     @staticmethod
     def confirm(action, do_not_confirm):
-        print(
-            "%s%s\nThis is an irreversible operation!"
-            % (action, "" if action.endswith("\n") else ". ")
+        log.print(
+            "{}{}\nThis is an irreversible operation!".format(
+                action, "" if action.endswith("\n") else ". "
+            )
         )
         if not do_not_confirm:
-            print("Type 'BURN' (all capitals) to continue.")
-            # required for Pythons which disable line buffering, ie mingw in mintty
-            sys.stdout.flush()
+            log.print("Type 'BURN' (all capitals) to continue.", flush=True)
+            # Flush required for Pythons which disable line buffering,
+            # ie mingw in mintty
             yes = input()
             if yes != "BURN":
-                print("Aborting.")
+                log.print("Aborting.")
                 sys.exit(0)
 
     def print_error_msg(self, error_msg):
@@ -641,7 +696,7 @@ class EspEfusesBase(object):
             if not self.force_write_always:
                 error_msg += "(use '--force-write-always' option to ignore it)"
         if self.force_write_always:
-            print(error_msg, "Skipped because '--force-write-always' option.")
+            log.print(error_msg, "Skipped because '--force-write-always' option.")
         else:
             raise esptool.FatalError(error_msg)
 
@@ -650,9 +705,54 @@ class EspEfusesBase(object):
         return self.blocks[block_num].num_errors, self.blocks[block_num].fail
 
     def is_efuses_incompatible_for_burn(self):
-        # Overwrite this function for a specific target if you want to check if a certain eFuse(s) can be burned.
+        # Overwrite this function for a specific target if you want to check if a
+        # certain eFuse(s) can be burned.
         return False
 
+    def get_major_chip_version(self):
+        try:
+            return self["WAFER_VERSION_MAJOR"].get()
+        except KeyError:
+            return 0
+
+    def get_minor_chip_version(self):
+        try:
+            return self["WAFER_VERSION_MINOR"].get()
+        except KeyError:
+            return 0
+
+    def get_chip_version(self):
+        return self.get_major_chip_version() * 100 + self.get_minor_chip_version()
+
+    def get_major_block_version(self):
+        try:
+            return self["BLK_VERSION_MAJOR"].get()
+        except KeyError:
+            return 0
+
+    def get_minor_block_version(self):
+        try:
+            return self["BLK_VERSION_MINOR"].get()
+        except KeyError:
+            return 0
+
+    def get_block_version(self):
+        return self.get_major_block_version() * 100 + self.get_minor_block_version()
+
+    def get_pkg_version(self):
+        try:
+            return self["PKG_VERSION"].get()
+        except KeyError:
+            return 0
+
+    @abstractmethod
+    def summary(self):
+        pass
+
+    @abstractmethod
+    def get_coding_scheme_warnings(self, silent: bool = False):
+        pass
+
 
 class EfuseFieldBase(EfuseProtectBase):
     def __init__(self, parent, param):
@@ -665,7 +765,7 @@ class EfuseFieldBase(EfuseProtectBase):
         self.read_disable_bit = param.read_disable_bit
         self.name = param.name
         self.efuse_class = param.class_type
-        self.efuse_type = param.type
+        self.efuse_type: str = param.type
         self.description = param.description
         self.dict_value = param.dictionary
         self.bit_len = param.bit_len
@@ -718,10 +818,9 @@ class EfuseFieldBase(EfuseProtectBase):
                 try:
                     return BitArray(self.efuse_type + "={}".format(new_value))
                 except CreationError as err:
-                    print(
-                        "New value '{}' is not suitable for {} ({})".format(
-                            new_value, self.name, self.efuse_type
-                        )
+                    log.print(
+                        f"New value '{new_value}' is not suitable for "
+                        f"{self.name} ({self.efuse_type})"
                     )
                     raise esptool.FatalError(err)
 
@@ -733,23 +832,22 @@ class EfuseFieldBase(EfuseProtectBase):
 
         if bitarray_new_value.len != bitarray_old_value.len:
             raise esptool.FatalError(
-                "For {} efuse, the length of the new value is wrong, "
-                "expected {} bits, was {} bits.".format(
-                    self.name, bitarray_old_value.len, bitarray_new_value.len
-                )
+                f"For {self.name} eFuse, the length of the new value is wrong, "
+                f"expected {bitarray_old_value.len} bits, "
+                f"was {bitarray_new_value.len} bits."
             )
         if (
             bitarray_new_value == bitarray_old_value
             or bitarray_new_value & self.get_bitstring() == bitarray_new_value
         ):
-            error_msg = "\tThe same value for {} ".format(self.name)
-            error_msg += "is already burned. Do not change the efuse."
-            print(error_msg)
+            error_msg = f"\tThe same value for {self.name} "
+            error_msg += "is already burned. Do not change the eFuse."
+            log.print(error_msg)
             bitarray_new_value.set(0)
         elif bitarray_new_value == self.get_bitstring(from_read=False):
             error_msg = "\tThe same value for {} ".format(self.name)
             error_msg += "is already prepared for the burn operation."
-            print(error_msg)
+            log.print(error_msg)
             bitarray_new_value.set(0)
         else:
             if self.name not in ["WR_DIS", "RD_DIS"]:
@@ -798,49 +896,72 @@ class EfuseFieldBase(EfuseProtectBase):
             self.fail = self.parent.blocks[self.block].fail
             self.num_errors = self.parent.blocks[self.block].num_errors
 
-    def get_raw(self, from_read=True):
-        """Return the raw (unformatted) numeric value of the efuse bits
+    def get_raw(self, from_read: bool = True) -> int | bytearray:
+        """Return the raw (unformatted) numeric value of the eFuse bits
 
         Returns a simple integer or (for some subclasses) a bitstring.
         type: int or bool -> int
         type: bytes -> bytearray
-        """
-        return self.get_bitstring(from_read).read(self.efuse_type)
 
-    def get(self, from_read=True):
-        """Get a formatted version of the efuse value, suitable for display
+        Args:
+            from_read: If True, read the eFuse value from the device.
+                If False, use the cached value.
+
+        Returns:
+            int | bytearray: The raw value of the eFuse
+        """
+        return self.get_bitstring(from_read).read(self.efuse_type)  # type: ignore
+
+    def get(self, from_read: bool = True) -> str | int | bytearray:
+        """Get a formatted version of the eFuse value, suitable for display
+
         type: int or bool -> int
         type: bytes -> string  "01 02 03 04 05 06 07 08 ... ".
         Byte order [0] ... [N]. dump regs: 0x04030201 0x08070605 ...
+
+        Args:
+            from_read: If True, read the eFuse value from the device.
+                If False, use the cached value.
+
+        Returns:
+            str | int | bytearray: The formatted version of the eFuse value
         """
         if self.efuse_type.startswith("bytes"):
-            return util.hexify(self.get_bitstring(from_read).bytes[::-1], " ")
+            return util.hexify(self.get_bitstring(from_read).bytes[::-1], " ")  # type: ignore
         else:
             return self.get_raw(from_read)
 
-    def get_meaning(self, from_read=True):
-        """Get the meaning of efuse from dict if possible, suitable for display"""
+    def get_meaning(self, from_read: bool = True) -> str | int | bytearray:
+        """Get the meaning of eFuse from dict if possible, suitable for display
+
+        Args:
+            from_read: If True, read the eFuse value from the device.
+                If False, use the cached value.
+
+        Returns:
+            str | int | bytearray: The meaning of the eFuse
+        """
         if self.dict_value:
             try:
-                return self.dict_value[self.get_raw(from_read)]
+                return self.dict_value[self.get_raw(from_read)]  # type: ignore
             except KeyError:
                 pass
         return self.get(from_read)
 
-    def get_bitstring(self, from_read=True):
+    def get_bitstring(self, from_read: bool = True) -> BitStream | Bits:
         if from_read:
             self.bitarray.pos = 0
             return self.bitarray
         else:
             field_len = self.bitarray.len
-            block = self.parent.blocks[self.block]
+            block: EfuseBlockBase = self.parent.blocks[self.block]
             block.wr_bitarray.pos = block.wr_bitarray.length - (
                 self.word * 32 + self.pos + field_len
             )
             return block.wr_bitarray.read(self.bitarray.len)
 
     def burn(self, new_value):
-        # Burn a efuse. Added for compatibility reason.
+        """Burn a eFuse. Added for compatibility reason."""
         self.save(new_value)
         self.parent.burn_all()
 
@@ -860,7 +981,7 @@ class EfuseFieldBase(EfuseProtectBase):
         return output
 
     def reset(self):
-        # resets a efuse that is prepared for burning
+        """Resets a eFuse that is prepared for burning"""
         bitarray_field = self.convert_to_bitstring(0)
         block = self.parent.blocks[self.block]
         wr_bitarray_temp = block.wr_bitarray.copy()
