@@ -223,16 +223,21 @@ class BaseCommands:
                 f"({self.CHIP_NAME} != {self.esp.CHIP_NAME})"
             )
 
+    @classmethod
+    def _close_port(
+        cls,
+        esp: esptool.ESPLoader | EmulateEfuseControllerBase | None,
+        external_esp: bool = False,
+    ) -> None:
+        """Close the serial port if it was opened internally."""
+        if esp is not None and not external_esp and isinstance(esp, esptool.ESPLoader):
+            esp._port.close()
+
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
-        if (
-            self.esp is not None
-            and not self.external_esp
-            and isinstance(self.esp, esptool.ESPLoader)
-        ):
-            self.esp._port.close()
+        self._close_port(self.esp, self.external_esp)
 
     ################################# CLI definitions #################################
 
@@ -994,8 +999,8 @@ class BaseCommands:
                             f"{efuse_name} must be readable, stop this operation!"
                         )
                 else:
-                    for block in self.efuses.Blocks.BLOCKS:
-                        block = self.efuses.Blocks.get(block)
+                    for block_tuple in self.efuses.Blocks.BLOCKS:
+                        block = self.efuses.Blocks.get(block_tuple)
                         if block.name == efuse_name and block.key_purpose is not None:
                             if not self.efuses[block.key_purpose].need_rd_protect(
                                 self.efuses[block.key_purpose].get()
