@@ -138,7 +138,7 @@ class ESP32ROM(ESPLoader):
             # the contents will be readable and 0.
             # If the flash encryption is enabled it is expected to have a valid
             # non-zero key. We break out on first occurrence of non-zero value
-            key_word = [0] * 7
+            key_word = [0] * 8  # FE key consists of 8 registers/words.
             for i in range(len(key_word)):
                 key_word[i] = self.read_efuse(14 + i)
                 # key is non-zero so break & return
@@ -189,6 +189,13 @@ class ESP32ROM(ESPLoader):
             rev >= 300 and efuses & self.EFUSE_RD_ABS_DONE_1_MASK
         )
 
+    def get_secure_boot_v1_enabled(self):
+        """
+        Returns True if Secure Boot V1 is enabled (ABS_DONE_0 eFuse set).
+        """
+        efuses = self.read_reg(self.EFUSE_RD_ABS_DONE_REG)
+        return bool(efuses & self.EFUSE_RD_ABS_DONE_0_MASK)
+
     def get_pkg_version(self):
         word3 = self.read_efuse(3)
         pkg_version = (word3 >> 9) & 0x07
